//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------

#property indicator_separate_window
#property indicator_buffers    6
#property indicator_color1     clrYellow
#property indicator_color2     clrAqua
#property indicator_color3     clrAqua
#property indicator_color4     clrLime
#property indicator_color5     clrRed
#property indicator_color6     clrRed
#property indicator_width1     3
#property indicator_minimum    0
#property indicator_maximum    100
#property indicator_levelcolor clrGold
#property strict


//
//
//
//
//

enum enTimeFrames
{
   tf_cu  = 0,              // Current time frame
   tf_m1  = PERIOD_M1,      // 1 minute
   tf_m5  = PERIOD_M5,      // 5 minutes
   tf_m15 = PERIOD_M15,     // 15 minutes
   tf_m30 = PERIOD_M30,     // 30 minutes
   tf_h1  = PERIOD_H1,      // 1 hour
   tf_h4  = PERIOD_H4,      // 4 hours
   tf_d1  = PERIOD_D1,      // Daily
   tf_w1  = PERIOD_W1,      // Weekly
   tf_mb1 = PERIOD_MN1,     // Monthly
   tf_cus = 12345678        // Custom time frame
};

extern enTimeFrames       TimeFrame             = tf_cu;             // Time frame
extern int                TimeFrameCustom       = 0;                 // Custom time frame to use (if custom time frame used)
extern ENUM_APPLIED_PRICE RsiPrice              = 0;                 // Rsi price to use
extern int                RsiPeriod             = 14;                // Rsi period to use
extern int                MaPeriod              = 21;                // Rsi ma period to use
extern ENUM_MA_METHOD     MaMethod              = 1;                 // Rsi ma method
extern int                koridor               = 5;                 // Distance from Rsi Ma
extern bool               alertsOn              = true;              // Turn alerts on?
extern bool               alertsOnCurrent       = false;             // Alerts on still opened bar?
extern bool               alertsMessage         = true;              // Alerts should show popup message?
extern bool               alertsSound           = false;             // Alerts should play a sound?
extern bool               alertsNotify          = false;             // Alerts should send notification?
extern bool               alertsEmail           = false;             // Alerts should send email?
extern string             soundFile             = "alert2.wav";      // Alert sound file type
extern bool               arrowsVisible         = false;             // Arrows visible?
extern string             arrowsIdentifier      = "tarzan Arrows1";  // Unique ID for arrows
extern double             arrowsUpperGap        = 0.5;               // Upper arrow gap
extern double             arrowsLowerGap        = 0.5;               // Lower arrow gap
extern color              arrowsUpColor         = clrLimeGreen;      // Up arrow color
extern color              arrowsDnColor         = clrOrange;         // Down arrow color
extern int                arrowsUpCode          = 241;               // Up arrow code
extern int                arrowsDnCode          = 242;               // Down arrow code
extern int                arrowsUpSize          = 2;                 // Up arrow size
extern int                arrowsDnSize          = 2;                 // Down arrow size
extern bool               arrowsOnFirst         = true;              // Arrows on first bar?
extern bool               drawDots              = true;              // Draw dots or line
extern double             levelOs               = 25;                // Oversold linelevel
extern double             levelOb               = 75;                // Overbought line level

//
//
//
//
//

double rsi[];
double upBand[];
double dnBand[];
double rsiMaDa[];
double rsiMaDb[];
double rsiMa[];
double trend[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(7);
   SetIndexBuffer(0,rsi); 
   SetIndexBuffer(1,upBand);
   SetIndexBuffer(2,dnBand); 
   SetIndexBuffer(3,rsiMa);    
   SetIndexBuffer(4,rsiMaDa);
   SetIndexBuffer(5,rsiMaDb);  
   SetIndexBuffer(6,trend);
   SetLevelValue(0,levelOs); 
   SetLevelValue(1,levelOb);
   
   if (drawDots) 
   {
     SetIndexStyle(3, DRAW_ARROW,0); SetIndexArrow(3, 110);
     SetIndexStyle(4, DRAW_ARROW,0); SetIndexArrow(4, 110);
     SetIndexStyle(5, DRAW_NONE);
   }
   else
   {
     SetIndexStyle(3, DRAW_LINE);
     SetIndexStyle(4, DRAW_LINE);
     SetIndexStyle(5, DRAW_LINE);
   }        
            
         RsiPeriod         = fmax(1,RsiPeriod);
         indicatorFileName = WindowExpertName();
         returnBars        = TimeFrame==-99; 
         if (TimeFrameCustom==0) TimeFrameCustom = MathMax(TimeFrameCustom,_Period);
         if (TimeFrame!=tf_cus)
               TimeFrame = MathMax(TimeFrame,_Period);
         else  TimeFrame = (enTimeFrames)TimeFrameCustom;
      
   IndicatorShortName(timeFrameToString(TimeFrame)+"  Tarzan ("+(string)RsiPeriod+","+(string)MaPeriod+")");
return(0);
}
int deinit()
{
   string lookFor       = arrowsIdentifier+":";
   int    lookForLength = StringLen(lookFor);
   for (int i=ObjectsTotal()-1; i>=0; i--)
   {
      string objectName = ObjectName(i);
         if (StringSubstr(objectName,0,lookForLength) == lookFor) ObjectDelete(objectName);
   }
return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
         limit = MathMin(Bars - counted_bars,Bars-1);
         if (returnBars) { rsi[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (TimeFrame==Period())
   {
   
     if (!drawDots) if (trend[limit] == -1) CleanPoint(limit,rsiMaDa,rsiMaDb);
     for (i=limit; i>=0; i--) rsi[i] = iRSI(NULL,0,RsiPeriod,RsiPrice,i);
     for (i=limit; i>=0; i--) 
     {
        rsiMa[i]  = iMAOnArray(rsi,0,MaPeriod,0,MaMethod,i);
        upBand[i] = rsiMa[i]+koridor;
        dnBand[i] = rsiMa[i]-koridor;
        rsiMaDa[i] = EMPTY_VALUE;
        rsiMaDb[i] = EMPTY_VALUE;
        trend[i]   = (i<Bars-1) ? (rsi[i]>rsiMa[i]) ? 1 : (rsi[i]<rsiMa[i]) ? -1 : trend[i+1] : 0; 
        
           if (trend[i]==-1) if    (drawDots) rsiMaDa[i] = rsiMa[i];
                             else  PlotPoint(i,rsiMaDa,rsiMaDb,rsiMa);
         
           //
           //
           //
           //
           //
       
           if (arrowsVisible)
           {
             string lookFor = arrowsIdentifier+":"+(string)Time[i]; ObjectDelete(lookFor);            
               if (i<Bars-1 && trend[i] != trend[i+1])
               {
                  if (trend[i] == 1) drawArrow(i,arrowsUpColor,arrowsUpCode,arrowsUpSize,false);
                  if (trend[i] ==-1) drawArrow(i,arrowsDnColor,arrowsDnCode,arrowsDnSize, true);
               }
            }
   }
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert("sloping up");
      else  doAlert("sloping down");       
   }  
   return(0);
   }
   
   //
   //
   //
   //
   //

   limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   if (trend[limit]==-1) CleanPoint(limit,rsiMaDa,rsiMaDb);
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         rsi[i]     = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,0,RsiPrice,RsiPeriod,MaPeriod,MaMethod,koridor,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,arrowsOnFirst,0,y);
         upBand[i]  = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,0,RsiPrice,RsiPeriod,MaPeriod,MaMethod,koridor,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,arrowsOnFirst,1,y);
         dnBand[i]  = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,0,RsiPrice,RsiPeriod,MaPeriod,MaMethod,koridor,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,arrowsOnFirst,2,y);
         rsiMa[i]   = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,0,RsiPrice,RsiPeriod,MaPeriod,MaMethod,koridor,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,arrowsOnFirst,3,y);
         trend[i]   = iCustom(NULL,TimeFrame,indicatorFileName,tf_cu,0,RsiPrice,RsiPeriod,MaPeriod,MaMethod,koridor,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,arrowsVisible,arrowsIdentifier,arrowsUpperGap,arrowsLowerGap,arrowsUpColor,arrowsDnColor,arrowsUpCode,arrowsDnCode,arrowsUpSize,arrowsDnSize,arrowsOnFirst,6,y);
         rsiMaDa[i] = EMPTY_VALUE;
         rsiMaDb[i] = EMPTY_VALUE;
         if (drawDots && trend[i]==-1) rsiMaDa[i] = rsiMa[i];  
   }
   if (!drawDots) for(i=limit; i>=0; i--) if (trend[i]== -1) PlotPoint(i,rsiMaDa,rsiMaDb,rsiMa);
return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i]; first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] = from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                          second[i] = EMPTY_VALUE; }
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void drawArrow(int i,color theColor,int theCode,int theSize,bool up)
{
   string name = arrowsIdentifier+":"+(string)Time[i];
   double gap  = iATR(NULL,0,20,i);   
   int    add  = 0; if (!arrowsOnFirst) add = TimeFrame*60-1;
   
      //
      //
      //
      //
      //

      ObjectCreate(name,OBJ_ARROW,0,Time[i]+add,0);
         ObjectSet(name,OBJPROP_ARROWCODE,theCode);
         ObjectSet(name,OBJPROP_COLOR,theColor);
         ObjectSet(name,OBJPROP_WIDTH,theSize);
         if (up)
               ObjectSet(name,OBJPROP_PRICE1,High[fmax(iBarShift(NULL,0,Time[i]+add),0)] + arrowsUpperGap * gap);
         else  ObjectSet(name,OBJPROP_PRICE1, Low[fmax(iBarShift(NULL,0,Time[i]+add),0)] - arrowsLowerGap * gap);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," ",timeFrameToString(_Period)," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Tarzan ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Tarzan "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}
